package cc.fascinated.bat.command.impl.server.channel;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import lombok.NonNull;
import net.dv8tion.jda.api.Permission;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.Channel;
import net.dv8tion.jda.api.entities.channel.concrete.TextChannel;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.stereotype.Component;

/**
 * @author Nick (okNick)
 */
@Component
@CommandInfo(name = "settopic", description = "Set the topic of a channel", requiredPermissions = Permission.MANAGE_CHANNEL)
public class SetTopicSubCommand extends BatSubCommand {
    public SetTopicSubCommand() {
        super.addOption(OptionType.STRING, "topic", "The topic to set", true);
        super.addOption(OptionType.CHANNEL, "channel", "The channel to set the topic of", false);
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        Channel target = interaction.getOption("channel") == null ? channel : interaction.getOption("channel").getAsChannel();
        if (!(target instanceof TextChannel textChannel)) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("<#%s> is not a text channel!".formatted(target.getId()))
                            .build())
                    .queue();
            return;
        }

        String topic = interaction.getOption("topic").getAsString();
        if (topic.length() > 1024) {
            interaction.replyEmbeds(EmbedUtils.errorEmbed()
                            .setDescription("The topic must be less than 1024 characters!")
                            .build())
                    .queue();
            return;
        }

        textChannel.getManager().setTopic(topic).queue();
        interaction.replyEmbeds(EmbedUtils.successEmbed()
                .setDescription("Successfully set the topic of <#%s> to: \"%s\"".formatted(textChannel.getId(), topic))
                .build()
        ).queue();
    }
}